/*
 * (!c) 2002 by Ighighi
 * Venezuela
 *
 * TODO:
 *   More options:
 *     pcap timeout
 *     pcap snaplen
 *     pcap savefile ?
 */

#define VERSION	"netdump 0.7 by Ighighi"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>

/* 
 * We need to know if we're on a 2.0.x kernel to avoid
 * duplicate packets on the loopback interface
 */
#ifdef __linux__
#include <sys/utsname.h>	/* uname() */
#endif

#include <netinet/in.h>		/* struct in_addr */
#include <arpa/inet.h>		/* inet_ntoa() */

#include "pcap.h"

pcap_handler lookup_handler(int datalink);

static void cleanup(int signum);
static void exit_usage(int status);
static void parse_args(int argc, char *argv[]);
static void print_stats(void);

#define SNAPLEN	0x10000

/*
 * Global variables
 */

#ifdef __linux__
int linux_loopback	= 0;	/* avoid printing loopback addresses */
int avoid_duplicate	= 0;	/* avoid duplicate packets */
#endif

/* options: */
int verbose = 0;	/* verbose output ? */
int numeric = 0;	/* numeric output ? */
int promisc = 0;	/* promiscuous mode ? */
int resolve = 0;	/* resolve address ? */

/* arguments: */
static char *device	= NULL;		/* interface */
static char *progname	= NULL;		/* program name */
static char *expression	= NULL;		/* bpf expression */
static int count	= -1;		/* max number of packets */

static pcap_t *pcap_d	= NULL;		/* pcap descriptor */

static void cleanup(int signum)
{
    if (pcap_d) {
	if (verbose) {
	    print_stats();
	}
        pcap_close(pcap_d);
    }

    if (verbose) {
	printf("Exiting on signal: %d\n", signum);
    }
    
    exit(0);
}

static void exit_usage(int status)
{
    FILE *fp;
    
    if (status)
	fp = stderr;
    else
	fp = stdout;
    
    if (! progname)
	return;

    fprintf(fp, "%s\n", VERSION);    
    fprintf(fp, "Usage: %s [options] [device] \n", progname);
    fprintf(fp, "   options: \n");
    fprintf(fp, "   -n     numeric output \n");
    fprintf(fp, "   -v     verbose output \n");
    fprintf(fp, "   -p     promiscous mode \n");
    fprintf(fp, "   -r     resolve addresses \n");
    fprintf(fp, "   -c count \n");
    fprintf(fp, "          exit after receiving count packets \n");
    fprintf(fp, "   -e expression \n");
    fprintf(fp, "          filter expression (see tcpdump(1)) \n");
    fprintf(fp, "   -h,    print this help and exit \n");
    fprintf(fp, "   -V,    print version and exit \n");
    
    exit(status);
    
    /* NOTREACH */
    return;
}

static void parse_args(int argc, char *argv[])
{
    int i;
    
    progname = argv[0];
    
    for (i = 1; i < argc; i++) {
	if (argv[i][0] == '-') {
	    switch (argv[i][1]) {
	    case 'n':
		numeric = 1;
		break;
	    case 'v':
		verbose = 1;
		break;
	    case 'p':
		promisc = 1;
		break;
	    case 'r':
		resolve = 1;
		break;
	    case 'c':
		if (! argv[++i])
		    exit_usage(1);
		count = atoi(argv[i]);
		break;
	    case 'e':
		if (! (expression = argv[++i]))
		    exit_usage(1);
		break;
	    case 'h':
		exit_usage(0);
		break;
	    case 'V':
		printf("%s\n", VERSION);
		exit(0);
		break;
	    default:
		fprintf(stderr, "%s: invalid option: %s\n", progname, argv[i]);
		exit_usage(1);
	    }
	}
	else {
	    break;
	}
    }

    if (i < argc) {
	device = argv[i++];
    }
    
    if (i != argc) {
	fprintf(stderr, "%s: too many arguments\n", progname);
	exit_usage(1);
    }
    
    return;
}

void print_stats(void)
{
    struct pcap_stat pcap_st;	/* pcap statistics */
    
    if (pcap_stats(pcap_d, &pcap_st) < 0) {
	pcap_perror(pcap_d, progname);
    }
    else {
	putchar('\n');
	printf("packets received: %u\n", pcap_st.ps_recv);
	printf("packets  dropped: %u\n", pcap_st.ps_drop);
    }
    
    return;
}

int main(int argc, char *argv[])
{
    char pcap_err[PCAP_ERRBUF_SIZE];	/* pcap error message */
    struct bpf_program fprog;		/* bpf filter program */
    pcap_handler callback;		/* callback function */
    int datalink;
    int snapshot;
    bpf_u_int32 net_addr, net_mask;

    /* No buffering on stdout & stderr */
    setbuf(stdout, NULL);
    setbuf(stderr, NULL);

    parse_args(argc, argv);

    signal(SIGHUP, SIG_IGN);	/* Hangup */
    signal(SIGINT, cleanup);	/* Interrupt from keyboard */
    signal(SIGQUIT, cleanup);	/* Quit from keyboard */
    signal(SIGTERM, cleanup);	/* Termination signal */    
    
    if (! device) {
	device = pcap_lookupdev(pcap_err);
	if (! device) {
	    fprintf(stderr, "%s: %s\n", progname, pcap_err);
	    goto bad;
	}
    }
    
#ifdef __linux__
    if (strcmp(device, "lo") == 0) {
	struct utsname uts;

	linux_loopback = 1;	
	
	if (uname(&uts) < 0) {
	    perror("uname()");
	    exit(1);
	}
	
	if (strncmp(uts.release, "2.0", 3) == 0) {
	    avoid_duplicate = 1;
	    count <<= 1;	/* double the number of packets */
	}
    }
#endif    

    if (pcap_lookupnet(device, &net_addr, &net_mask, pcap_err) < 0) {
	fprintf(stderr, "%s: %s\n", progname, pcap_err);
	goto bad;
    }
    
    pcap_d = pcap_open_live(device, SNAPLEN, promisc, 0, pcap_err);
    if (! pcap_d) {
	fprintf(stderr, "%s: %s\n", progname, pcap_err);
	goto bad;
    }

    snapshot = pcap_snapshot(pcap_d);
    if (snapshot < 0) {
	pcap_perror(pcap_d, progname);
	goto bad;
    }	
    
    datalink = pcap_datalink(pcap_d);
    if (datalink < 0) {
	pcap_perror(pcap_d, progname);
	goto bad;
    }

    callback = lookup_handler(datalink);
    if (! callback) {
	fprintf(stderr, "%s: Unknown data-link type: 0x%0x\n",
		progname, datalink);
	goto bad;
    }

    if (expression) {
	if (pcap_compile(pcap_d, &fprog, expression, 1, net_mask) < 0) {
	    fprintf(stderr, "%s: expression: %s\n",
		    progname, pcap_geterr(pcap_d));
	    goto bad;
	}
	if (pcap_setfilter(pcap_d, &fprog) < 0) {
	    fprintf(stderr, "%s: %s\n", progname, pcap_geterr(pcap_d));
	    goto bad;
	}	    
    }
    
    if (verbose) {
	printf("device=%s ", device);
	/* TODO: Translate into a string */
	printf("dlt=%d ", datalink);
	printf("addr=%s ", inet_ntoa(*(struct in_addr *)&net_addr));
	printf("mask=%s ", inet_ntoa(*(struct in_addr *)&net_mask));
	printf("size=%d\n", snapshot);	
    }

    if (pcap_loop(pcap_d, count, callback, NULL) < 0) {
	pcap_perror(pcap_d, progname);
	goto bad;
    }

    if (verbose) {
	print_stats();
    }
    
    if (pcap_d) {
	pcap_close(pcap_d);
    }
    
    exit(0);
    
bad:
    if (pcap_d) {
	pcap_close(pcap_d);
    }
    exit(1);    
}
